@@ -24,6 +24,9 @@
 #ifndef __GST_CLOCK_H__
 #define __GST_CLOCK_H__
 
+#include <gst/gstconfig.h>
+#include <glib.h>
+
 G_BEGIN_DECLS
 
 /* --- standard type macros --- */
@@ -63,12 +66,9 @@ typedef gint64 GstClockTimeDiff;
 typedef gpointer GstClockID;
 
 /**
- * GST_CLOCK_TIME_NONE:
+ * GST_CLOCK_TIME_NONE: (value 18446744073709551615) (type GstClockTime)
  *
  * Constant to define an undefined clock time.
- *
- * Value: 18446744073709551615
- * Type: GstClockTime
  */
 #define GST_CLOCK_TIME_NONE             ((GstClockTime) -1)
 /**
@@ -79,43 +79,46 @@ typedef gpointer GstClockID;
  */
 #define GST_CLOCK_TIME_IS_VALID(time)   (((GstClockTime)(time)) != GST_CLOCK_TIME_NONE)
 
-/* FIXME: still need to explicitly force types on the defines below */
 /**
- * GST_SECOND:
+ * GST_CLOCK_STIME_NONE: (value -9223372036854775808) (type GstClockTimeDiff)
+ *
+ * Constant to define an undefined clock time.
+ */
+#define GST_CLOCK_STIME_NONE             ((GstClockTimeDiff)G_MININT64)
+/**
+ * GST_CLOCK_STIME_IS_VALID:
+ * @time: signed clock time to validate
+ *
+ * Tests if a given #GstClockTimeDiff of #gint64 represents a valid defined time.
+ *
+ * Since: 1.6
+ */
+#define GST_CLOCK_STIME_IS_VALID(time)   (((GstClockTimeDiff)(time)) != GST_CLOCK_STIME_NONE)
+
+/**
+ * GST_SECOND: (value 1000000000) (type GstClockTimeDiff)
  *
  * Constant that defines one GStreamer second.
- *
- * Value: 1000000000
- * Type: GstClockTime
  */
-#define GST_SECOND  (G_USEC_PER_SEC * G_GINT64_CONSTANT (1000))
+#define GST_SECOND  ((GstClockTimeDiff)(G_USEC_PER_SEC * G_GINT64_CONSTANT (1000)))
 /**
- * GST_MSECOND:
+ * GST_MSECOND: (value 1000000) (type GstClockTimeDiff)
  *
  * Constant that defines one GStreamer millisecond.
- *
- * Value: 1000000
- * Type: GstClockTime
  */
-#define GST_MSECOND (GST_SECOND / G_GINT64_CONSTANT (1000))
+#define GST_MSECOND ((GstClockTimeDiff)(GST_SECOND / G_GINT64_CONSTANT (1000)))
 /**
- * GST_USECOND:
+ * GST_USECOND: (value 1000) (type GstClockTimeDiff)
  *
  * Constant that defines one GStreamer microsecond.
- *
- * Value: 1000
- * Type: GstClockTime
  */
-#define GST_USECOND (GST_SECOND / G_GINT64_CONSTANT (1000000))
+#define GST_USECOND ((GstClockTimeDiff)(GST_SECOND / G_GINT64_CONSTANT (1000000)))
 /**
- * GST_NSECOND:
+ * GST_NSECOND: (value 1) (type GstClockTimeDiff)
  *
  * Constant that defines one GStreamer nanosecond
- *
- * Value: 1
- * Type: GstClockTime
  */
-#define GST_NSECOND (GST_SECOND / G_GINT64_CONSTANT (1000000000))
+#define GST_NSECOND ((GstClockTimeDiff)(GST_SECOND / G_GINT64_CONSTANT (1000000000)))
 
 
 /**
@@ -172,9 +175,9 @@ typedef gpointer GstClockID;
  *
  * Convert a #GstClockTime to a #GTimeVal
  *
- * <note>on 32-bit systems, a timeval has a range of only 2^32 - 1 seconds,
- * which is about 68 years.  Expect trouble if you want to schedule stuff
- * in your pipeline for 2038.</note>
+ * > on 32-bit systems, a timeval has a range of only 2^32 - 1 seconds,
+ * > which is about 68 years.  Expect trouble if you want to schedule stuff
+ * > in your pipeline for 2038.
  */
 #define GST_TIME_TO_TIMEVAL(t,tv)                               \
 G_STMT_START {                                                  \
@@ -217,7 +220,7 @@ G_STMT_START {                                                    \
  * the matching arguments.
  *
  * Example:
- * |[
+ * |[<!-- language="C" -->
  * printf("%" GST_TIME_FORMAT "\n", GST_TIME_ARGS(ts));
  * ]|
  */
@@ -238,6 +241,40 @@ G_STMT_START {                                                    \
         (guint) ((((GstClockTime)(t)) / GST_SECOND) % 60) : 99, \
         GST_CLOCK_TIME_IS_VALID (t) ? \
         (guint) (((GstClockTime)(t)) % GST_SECOND) : 999999999
+/**
+ * GST_STIME_FORMAT:
+ *
+ * A string that can be used in printf-like format strings to display a signed
+ * #GstClockTimeDiff or #gint64 value in h:m:s format.  Use GST_TIME_ARGS() to
+ * construct the matching arguments.
+ *
+ * Example:
+ * |[
+ * printf("%" GST_STIME_FORMAT "\n", GST_STIME_ARGS(ts));
+ * ]|
+ *
+ * Since: 1.6
+ */
+#define GST_STIME_FORMAT "c%" GST_TIME_FORMAT
+/**
+ * GST_STIME_ARGS:
+ * @t: a #GstClockTimeDiff or #gint64
+ *
+ * Format @t for the #GST_STIME_FORMAT format string. Note: @t will be
+ * evaluated more than once.
+ *
+ * Since: 1.6
+ */
+#define GST_STIME_ARGS(t)						\
+  ((t) == GST_CLOCK_STIME_NONE || (t) >= 0) ? '+' : '-',		\
+    GST_CLOCK_STIME_IS_VALID (t) ?					\
+    (guint) (((GstClockTime)(ABS(t))) / (GST_SECOND * 60 * 60)) : 99,	\
+    GST_CLOCK_STIME_IS_VALID (t) ?					\
+    (guint) ((((GstClockTime)(ABS(t))) / (GST_SECOND * 60)) % 60) : 99,	\
+    GST_CLOCK_STIME_IS_VALID (t) ?					\
+    (guint) ((((GstClockTime)(ABS(t))) / GST_SECOND) % 60) : 99,	\
+    GST_CLOCK_STIME_IS_VALID (t) ?					\
+    (guint) (((GstClockTime)(ABS(t))) % GST_SECOND) : 999999999
 
 typedef struct _GstClockEntry   GstClockEntry;
 typedef struct _GstClock        GstClock;
@@ -375,6 +412,8 @@ struct _GstClockEntry {
  * @GST_CLOCK_FLAG_CAN_DO_PERIODIC_ASYNC: clock can do async periodic timeout callbacks
  * @GST_CLOCK_FLAG_CAN_SET_RESOLUTION: clock's resolution can be changed
  * @GST_CLOCK_FLAG_CAN_SET_MASTER: clock can be slaved to a master clock
+ * @GST_CLOCK_FLAG_NEEDS_STARTUP_SYNC: clock needs to be synced before it can be used
+ *     (Since 1.6)
  * @GST_CLOCK_FLAG_LAST: subclasses can add additional flags starting from this flag
  *
  * The capabilities of this clock
@@ -386,6 +425,7 @@ typedef enum {
   GST_CLOCK_FLAG_CAN_DO_PERIODIC_ASYNC  = (GST_OBJECT_FLAG_LAST << 3),
   GST_CLOCK_FLAG_CAN_SET_RESOLUTION     = (GST_OBJECT_FLAG_LAST << 4),
   GST_CLOCK_FLAG_CAN_SET_MASTER         = (GST_OBJECT_FLAG_LAST << 5),
+  GST_CLOCK_FLAG_NEEDS_STARTUP_SYNC     = (GST_OBJECT_FLAG_LAST << 6),
   /* padding */
   GST_CLOCK_FLAG_LAST                   = (GST_OBJECT_FLAG_LAST << 8)
 } GstClockFlags;
@@ -451,71 +491,143 @@ struct _GstClockClass {
   gpointer _gst_reserved[GST_PADDING];
 };
 
+GST_API
 GType                   gst_clock_get_type              (void);
 
+GST_API
 GstClockTime            gst_clock_set_resolution        (GstClock *clock,
                                                          GstClockTime resolution);
+GST_API
 GstClockTime            gst_clock_get_resolution        (GstClock *clock);
 
+GST_API
 GstClockTime            gst_clock_get_time              (GstClock *clock);
+
+GST_API
 void                    gst_clock_set_calibration       (GstClock *clock, GstClockTime internal,
                                                          GstClockTime external,
                                                          GstClockTime rate_num,
                                                          GstClockTime rate_denom);
+GST_API
 void                    gst_clock_get_calibration       (GstClock *clock, GstClockTime *internal,
                                                          GstClockTime *external,
                                                          GstClockTime *rate_num,
                                                          GstClockTime *rate_denom);
 
 /* master/slave clocks */
+
+GST_API
 gboolean                gst_clock_set_master            (GstClock *clock, GstClock *master);
+
+GST_API
 GstClock*               gst_clock_get_master            (GstClock *clock);
 
+GST_API
 void                    gst_clock_set_timeout           (GstClock *clock,
                                                          GstClockTime timeout);
+GST_API
 GstClockTime            gst_clock_get_timeout           (GstClock *clock);
 
+GST_API
 gboolean                gst_clock_add_observation       (GstClock *clock, GstClockTime slave,
                                                          GstClockTime master, gdouble *r_squared);
-
+GST_API
+gboolean                gst_clock_add_observation_unapplied (GstClock *clock, GstClockTime slave,
+                                                         GstClockTime master, gdouble *r_squared,
+                                                         GstClockTime *internal,
+                                                         GstClockTime *external,
+                                                         GstClockTime *rate_num,
+                                                         GstClockTime *rate_denom);
 
 /* getting and adjusting internal/external time */
+
+GST_API
 GstClockTime            gst_clock_get_internal_time     (GstClock *clock);
+
+GST_API
 GstClockTime            gst_clock_adjust_unlocked       (GstClock *clock, GstClockTime internal);
+
+GST_API
+GstClockTime            gst_clock_adjust_with_calibration (GstClock *clock,
+                                                         GstClockTime internal_target,
+                                                         GstClockTime cinternal,
+                                                         GstClockTime cexternal,
+                                                         GstClockTime cnum,
+                                                         GstClockTime cdenom);
+GST_API
+GstClockTime            gst_clock_unadjust_with_calibration (GstClock *clock,
+                                                         GstClockTime external_target,
+                                                         GstClockTime cinternal,
+                                                         GstClockTime cexternal,
+                                                         GstClockTime cnum,
+                                                         GstClockTime cdenom);
+GST_API
 GstClockTime            gst_clock_unadjust_unlocked     (GstClock * clock, GstClockTime external);
 
+/* waiting for, signalling and checking for synchronization */
+
+GST_API
+gboolean                gst_clock_wait_for_sync         (GstClock * clock, GstClockTime timeout);
+
+GST_API
+gboolean                gst_clock_is_synced             (GstClock * clock);
+
+/* to be used by subclasses only */
+
+GST_API
+void                    gst_clock_set_synced            (GstClock * clock, gboolean synced);
 
 /* creating IDs that can be used to get notifications */
+
+GST_API
 GstClockID              gst_clock_new_single_shot_id    (GstClock *clock,
                                                          GstClockTime time);
+GST_API
 GstClockID              gst_clock_new_periodic_id       (GstClock *clock,
                                                          GstClockTime start_time,
                                                          GstClockTime interval);
 
 /* reference counting */
+
+GST_API
 GstClockID              gst_clock_id_ref                (GstClockID id);
+
+GST_API
 void                    gst_clock_id_unref              (GstClockID id);
 
 /* operations on IDs */
+
+GST_API
 gint                    gst_clock_id_compare_func       (gconstpointer id1, gconstpointer id2);
 
+GST_API
 GstClockTime            gst_clock_id_get_time           (GstClockID id);
+
+GST_API
 GstClockReturn          gst_clock_id_wait               (GstClockID id,
                                                          GstClockTimeDiff *jitter);
+GST_API
 GstClockReturn          gst_clock_id_wait_async         (GstClockID id,
                                                          GstClockCallback func,
                                                          gpointer user_data,
                                                          GDestroyNotify destroy_data);
+GST_API
 void                    gst_clock_id_unschedule         (GstClockID id);
 
+GST_API
 gboolean                gst_clock_single_shot_id_reinit (GstClock * clock,
                                                          GstClockID id,
                                                          GstClockTime time);
+GST_API
 gboolean                gst_clock_periodic_id_reinit    (GstClock * clock,
                                                          GstClockID id,
                                                          GstClockTime start_time,
                                                          GstClockTime interval);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstClock, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_CLOCK_H__ */
