@@ -52,7 +52,7 @@ G_BEGIN_DECLS
  * @GST_PAD_ALWAYS: the pad is always available
  * @GST_PAD_SOMETIMES: the pad will become available depending on the media stream
  * @GST_PAD_REQUEST: the pad is only available on request with
- *  gst_element_get_request_pad().
+ *  gst_element_request_pad().
  *
  * Indicates when this pad will become available.
  */
@@ -94,6 +94,16 @@ typedef enum {
  */
 #define GST_PAD_TEMPLATE_CAPS(templ)		(((GstPadTemplate *)(templ))->caps)
 
+/**
+ * GST_PAD_TEMPLATE_GTYPE:
+ * @templ: the template to query
+ *
+ * Get the #GType of the padtemplate
+ *
+ * Since: 1.14
+ */
+#define GST_PAD_TEMPLATE_GTYPE(templ)		(((GstPadTemplate *)(templ))->ABI.abi.gtype)
+
 /**
  * GstPadTemplateFlags:
  * @GST_PAD_TEMPLATE_FLAG_LAST: first flag that can be used by subclasses.
@@ -127,7 +137,12 @@ struct _GstPadTemplate {
   GstCaps	  *caps;
 
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  union {
+    gpointer _gst_reserved[GST_PADDING];
+    struct {
+      GType gtype;
+    } abi;
+  } ABI;
 };
 
 struct _GstPadTemplateClass {
@@ -175,19 +190,42 @@ struct _GstStaticPadTemplate {
 }
 
 /* templates and factories */
+
+GST_API
 GType			gst_pad_template_get_type		(void);
+
+GST_API
 GType			gst_static_pad_template_get_type	(void);
 
+GST_API
 GstPadTemplate*		gst_pad_template_new			(const gchar *name_template,
 								 GstPadDirection direction, GstPadPresence presence,
 								 GstCaps *caps) G_GNUC_MALLOC;
-
+GST_API
+GstPadTemplate*		gst_pad_template_new_with_gtype		(const gchar *name_template,
+								 GstPadDirection direction, GstPadPresence presence,
+								 GstCaps *caps, GType pad_type) G_GNUC_MALLOC;
+GST_API
 GstPadTemplate *	gst_static_pad_template_get             (GstStaticPadTemplate *pad_template);
+
+GST_API
+GstPadTemplate * gst_pad_template_new_from_static_pad_template_with_gtype (
+    GstStaticPadTemplate * pad_template,
+    GType pad_type);
+
+GST_API
 GstCaps*		gst_static_pad_template_get_caps	(GstStaticPadTemplate *templ);
+
+GST_API
 GstCaps*		gst_pad_template_get_caps		(GstPadTemplate *templ);
 
+GST_API
 void                    gst_pad_template_pad_created            (GstPadTemplate * templ, GstPad * pad);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstPadTemplate, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_PAD_TEMPLATE_H__ */
