@@ -1092,14 +1092,18 @@ update_ptp_time (PtpDomainData * domain, PtpPendingSync * sync)
 
 #ifdef USE_MEASUREMENT_FILTERING
   /* We check this here and when updating the mean path delay, because
-   * we can get here without a delay response too */
+   * we can get here without a delay response too. The tolerance on
+   * accepting follow-up after a sync is high, because a PTP server
+   * doesn't have to prioritise sending FOLLOW_UP - its purpose is
+   * just to give us the accurate timestamp of the preceding SYNC */
   if (sync->follow_up_recv_time_local != GST_CLOCK_TIME_NONE
       && sync->follow_up_recv_time_local >
-      sync->sync_recv_time_local + 2 * domain->mean_path_delay) {
-    GST_WARNING ("Sync-follow-up delay for domain %u too big: %" GST_TIME_FORMAT
-        " > 2 * %" GST_TIME_FORMAT, domain->domain,
-        GST_TIME_ARGS (sync->follow_up_recv_time_local),
-        GST_TIME_ARGS (domain->mean_path_delay));
+      sync->sync_recv_time_local + 20 * domain->mean_path_delay) {
+    GstClockTimeDiff delay =
+        sync->follow_up_recv_time_local - sync->sync_recv_time_local;
+    GST_WARNING ("Sync-follow-up delay for domain %u too big: %"
+        GST_STIME_FORMAT " > 20 * %" GST_TIME_FORMAT, domain->domain,
+        GST_STIME_ARGS (delay), GST_TIME_ARGS (domain->mean_path_delay));
     synced = FALSE;
     gst_clock_get_calibration (GST_CLOCK_CAST (domain->domain_clock),
         &internal_time, &external_time, &rate_num, &rate_den);
@@ -1357,12 +1361,15 @@ update_mean_path_delay (PtpDomainData * domain, PtpPendingSync * sync)
 #endif
 
 #ifdef USE_MEASUREMENT_FILTERING
+  /* The tolerance on accepting follow-up after a sync is high, because
+   * a PTP server doesn't have to prioritise sending FOLLOW_UP - its purpose is
+   * just to give us the accurate timestamp of the preceding SYNC */
   if (sync->follow_up_recv_time_local != GST_CLOCK_TIME_NONE &&
       domain->mean_path_delay != 0
       && sync->follow_up_recv_time_local >
-      sync->sync_recv_time_local + 2 * domain->mean_path_delay) {
+      sync->sync_recv_time_local + 20 * domain->mean_path_delay) {
     GST_WARNING ("Sync-follow-up delay for domain %u too big: %" GST_TIME_FORMAT
-        " > 2 * %" GST_TIME_FORMAT, domain->domain,
+        " > 20 * %" GST_TIME_FORMAT, domain->domain,
         GST_TIME_ARGS (sync->follow_up_recv_time_local -
             sync->sync_recv_time_local),
         GST_TIME_ARGS (domain->mean_path_delay));
@@ -1384,10 +1391,14 @@ update_mean_path_delay (PtpDomainData * domain, PtpPendingSync * sync)
       sync->delay_resp_recv_time_local - sync->delay_req_send_time_local;
 
 #ifdef USE_MEASUREMENT_FILTERING
-  /* delay_req_delay is a RTT, so 2 times the path delay */
-  if (delay_req_delay > 4 * domain->mean_path_delay) {
+  /* delay_req_delay is a RTT, so 2 times the path delay is what we'd
+   * hope for, but some PTP systems don't prioritise sending DELAY_RESP,
+   * but they must still have placed an accurate reception timestamp.
+   * That means we should be quite tolerant about late DELAY_RESP, and
+   * mostly rely on filtering out jumps in the mean-path-delay elsewhere  */
+  if (delay_req_delay > 20 * domain->mean_path_delay) {
     GST_WARNING ("Delay-request-response delay for domain %u too big: %"
-        GST_TIME_FORMAT " > 4 * %" GST_TIME_FORMAT, domain->domain,
+        GST_TIME_FORMAT " > 20 * %" GST_TIME_FORMAT, domain->domain,
         GST_TIME_ARGS (delay_req_delay),
         GST_TIME_ARGS (domain->mean_path_delay));
     ret = FALSE;
@@ -2371,8 +2382,9 @@ gst_ptp_clock_ensure_domain_clock (GstPtpClock * self)
       for (l = domain_clocks; l; l = l->next) {
         PtpDomainData *clock_data = l->data;
 
-        if (clock_data->domain == self->priv->domain
-            && clock_data->last_ptp_time != 0) {
+        if (clock_data->domain == self->priv->domain &&
+            clock_data->have_master_clock && clock_data->last_ptp_time != 0) {
+          GST_DEBUG ("Switching domain clock on domain %d", clock_data->domain);
           self->priv->domain_clock = clock_data->domain_clock;
           got_clock = TRUE;
           break;
