@@ -28,7 +28,16 @@
 G_BEGIN_DECLS
 
 G_GNUC_INTERNAL
-char *    gst_buffer_get_flags_string                   (GstBuffer *buffer);
+gchar *   gst_buffer_get_flags_string (GstBuffer *buffer);
+
+G_GNUC_INTERNAL
+gchar *   gst_buffer_get_meta_string (GstBuffer * buffer);
+
+G_GNUC_INTERNAL
+GstFlowReturn  gst_writev_buffers (GstObject * sink, gint fd, GstPoll * fdset,
+                                   GstBuffer ** buffers, guint num_buffers,
+                                   guint8 * mem_nums, guint total_mem_num,
+                                   guint64 * bytes_written, guint64 skip);
 
 G_END_DECLS
 
