@@ -1,3 +1,27 @@
+/* GStreamer
+ *
+ * some unit tests for GstBaseTransform
+ *
+ * Copyright (C) 2008 Wim Taymans <wim.taymans@gmail.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/base/gstbasetransform.h>
 
@@ -28,18 +52,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 typedef struct _GstTestTrans GstTestTrans;
 typedef struct _GstTestTransClass GstTestTransClass;
 
-#define GST_TYPE_TEST_TRANS \
-  (gst_test_trans_get_type())
-#define GST_TEST_TRANS(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_TEST_TRANS,GstTestTrans))
-#define GST_TEST_TRANS_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_TEST_TRANS,GstTestTransClass))
-#define GST_TEST_TRANS_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS((obj), GST_TYPE_TEST_TRANS, GstTestTransClass))
-#define GST_IS_TEST_TRANS(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_TEST_TRANS))
-#define GST_IS_TEST_TRANS_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_TEST_TRANS))
+#define GST_TEST_TRANS(obj) ((GstTestTrans *)(obj))
 
 struct _GstTestTrans
 {
@@ -53,10 +66,6 @@ struct _GstTestTransClass
   GstBaseTransformClass parent_class;
 };
 
-GType gst_test_trans_get_type (void);
-
-G_DEFINE_TYPE (GstTestTrans, gst_test_trans, GST_TYPE_BASE_TRANSFORM);
-
 static GstFlowReturn (*klass_transform) (GstBaseTransform * trans,
     GstBuffer * inbuf, GstBuffer * outbuf) = NULL;
 static GstFlowReturn (*klass_transform_ip) (GstBaseTransform * trans,
@@ -89,10 +98,10 @@ gst_test_trans_class_init (GstTestTransClass * klass)
   gst_element_class_set_metadata (element_class, "TestTrans",
       "Filter/Test", "Test transform", "Wim Taymans <wim.taymans@gmail.com>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (src_template));
+  gst_element_class_add_static_pad_template (element_class, sink_template);
+  gst_element_class_add_static_pad_template (element_class, src_template);
+
+  GST_INFO ("setting up %s", g_type_name (((GTypeClass *) klass)->g_type));
 
   trans_class->passthrough_on_same_caps = klass_passthrough_on_same_caps;
   if (klass_transform_ip != NULL)
@@ -158,9 +167,24 @@ gst_test_trans_new (void)
   TestTransData *res;
   GstPad *tmp;
   GstPadTemplate *templ;
+  GType type;
+
+  /* we register a new sub-class for every test-run, so the class init
+   * function is called for every test run and can be set up properly
+   * even with CK_FORK=no */
+  {
+    static gint counter = 0;
+    gchar name[100];
+
+    g_snprintf (name, sizeof (name), "GstTestTrans%d", ++counter);
+
+    type = g_type_register_static_simple (GST_TYPE_BASE_TRANSFORM, name,
+        sizeof (GstTestTransClass), (GClassInitFunc) gst_test_trans_class_init,
+        sizeof (GstTestTrans), (GInstanceInitFunc) gst_test_trans_init, 0);
+  }
 
   res = g_new0 (TestTransData, 1);
-  res->trans = g_object_new (GST_TYPE_TEST_TRANS, NULL);
+  res->trans = g_object_new (type, NULL);
 
   templ = gst_static_pad_template_get (sink_template);
   templ->direction = GST_PAD_SRC;
